"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const passport_strategy_runner_1 = require("passport-strategy-runner");
function isPassportAuthenticator(obj) {
    return obj.use && obj.authenticate && obj.initialize && obj.session;
}
function isString(obj) {
    return typeof (obj) === 'string';
}
/* istanbul ignore next */
function assertNever(x) {
    throw new Error("Unexpected object: " + JSON.stringify(x));
}
function defaultConverter(user) {
    return {
        user: user,
        roles: user.roles,
        scopes: user.scopes
    };
}
/**
 * Create a new Exegesis authenticator from a passport instance.
 *
 * @param passport - The passport instance to call into.  This should be installed
 *   in your middleware chain before exegesis with `app.use(passport.initialize())`.
 * @param strategyName - The name of the passport strategy to use.  This strategy
 *   should be registered with passport with `passport.use(...)`.
 * @param converter - A function to convert Passport users into
 *   `{user, roles, scopes}` objects.  If not defined, `user.roles` will be used
 *   as roles, and `user.scopes` will be used as scopes.
 * @returns - An Exegesis authenticator.
 */
function makePassportAuthenticator(passport, strategyName, converter = defaultConverter) {
    return function passportAuthenticator(pluginContext, done) {
        passport.authorize(strategyName, (err, user, challenge, status) => {
            if (err) {
                done(err);
            }
            else if (user) {
                const result = converter(user, pluginContext);
                if (!result.user) {
                    result.user = user;
                }
                done(null, result);
            }
            else {
                done(null, { type: 'fail', challenge, status });
            }
        })(pluginContext.req, pluginContext.origRes, done);
    };
}
/**
 * Create a new Exegesis authenticator from a passport strategy.
 *
 * @param strategy - An instance of a Passport strategy to call.  Passport
 *   does not need to be registered as a middleware for this to work.
 * @param converter - A function to convert Passport users into
 *   `{user, roles, scopes}` objects.  If not defined, `user.roles` will be used
 *   as roles, and `user.scopes` will be used as scopes.
 * @returns - An Exegesis authenticator.
 */
function makeStrategyRunner(strategy, converter = defaultConverter) {
    return function passportStrategyAuthenticator(pluginContext, done) {
        passport_strategy_runner_1.runStrategy(strategy, pluginContext.req, (err, result) => {
            if (err || !result) {
                return done(err);
            }
            switch (result.type) {
                case 'success': {
                    const answer = converter(result.user, pluginContext);
                    if (!answer.user) {
                        answer.user = result.user;
                    }
                    done(null, answer);
                    break;
                }
                case 'pass':
                    done(null, undefined);
                    break;
                case 'fail':
                    done(null, result);
                    break;
                case 'redirect': {
                    const { res } = pluginContext;
                    res.setStatus(result.status)
                        .set('location', result.url)
                        .setBody('');
                    done(null, undefined);
                    break;
                }
                /* istanbul ignore next */
                default:
                    assertNever(result);
            }
        });
    };
}
function exegesisPassport(a, b, converter) {
    if (isPassportAuthenticator(a) && isString(b)) {
        return makePassportAuthenticator(a, b, converter);
    }
    else {
        return makeStrategyRunner(a, b);
    }
}
exports.exegesisPassport = exegesisPassport;
exports.default = exegesisPassport;
//# sourceMappingURL=index.js.map